/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/

/*******************************************************************************
* File Name    : flipflop.h
* Version      : Applilet EZ PL for RL78
* Device(s)    : R5F1056A,R5F1057A,R5F1058A
* Tool-Chain   : CC-RL
* Description  : This file implements main function.
* Creation Date: 2016/09/14
*******************************************************************************/

#ifndef _FLIPFLOP_H_
#define _FLIPFLOP_H_
/*******************************************************************************
Includes <System Includes> , "Project Includes"
*******************************************************************************/

/*******************************************************************************
Macro definitions
*******************************************************************************/
/*******************************************************************************
** RS-FF	
********************************************************************************/
#define	RS_FF_INIT0																\
{																				\
		RSFF_Initialize();														\
}

#define	RS_FF_INIT																\
{																				\
		RSFF_Counter_Clear();													\
}

#define	RS_FF_PROC(Reset, Set, Out)												\
{																				\
	{																			\
		UCHAR	ucret;															\
																				\
		ucret = RSFF_Proc(Set, Reset);											\
		Out = ucret & 0x01;														\
		RSFF_Counter();															\
	}																			\
}

/*******************************************************************************
** D-FF
********************************************************************************/
#define	D_FF_INIT0																\
{																				\
		DFF_Initialize();														\
}

#define	D_FF_INIT																\
{																				\
		DFF_Counter_Clear();													\
}

#define	D_FF_PROC(Din, CKin, Out)												\
{																				\
		Out = DFF_Proc(Din, CKin);												\
		DFF_Counter( );															\
}

/*******************************************************************************
** T-FF
********************************************************************************/
#define	T_FF_INIT0																\
{																				\
		TFF_Initialize();														\
}

#define	T_FF_INIT																\
{																				\
		TFF_Counter_Clear();													\
}

#define	T_FF_PROC(Tin, Reset, Out)												\
{																				\
		Out = TFF_Proc(Tin, Reset);												\
		TFF_Counter( );															\
}

/*******************************************************************************
** JK-FF
********************************************************************************/
#define	JKFF_INIT0(init)														\
{																				\
		JKFF_Initialize(init);													\
		JKFF_Counter();															\
}

#define	JKFF_INIT																\
{																				\
		JKFF_Counter_Clear();													\
}

#define	JKFF_PROC(Jin, Clk, Kin, Qout, Qout2)									\
{																				\
	{																			\
		UCHAR	ucret;															\
		ucret = JKFF_Evaluate(Jin, Clk, Kin);									\
		Qout = ucret & 0x01;													\
		Qout2 = ~Qout & 0x01;													\
		JKFF_Counter();															\
	}																			\
}

/*******************************************************************************
** RS-FF2
********************************************************************************/
#define	RSFF2_INIT0(init)														\
{																				\
		RSFF2_Initialize(init);													\
		RSFF2_Counter();														\
}

#define	RSFF2_INIT																\
{																				\
		RSFF2_Counter_Clear();													\
}

#define	RSFF2_PROC(Sin, Rin, Out, Out2)	\
{																				\
	{																			\
		UCHAR	ucret;															\
		ucret = RSFF2_Evaluate(Sin, Rin);										\
		Out = ucret & 0x01;														\
		Out2 = ~Out & 0x01;														\
		RSFF2_Counter();														\
	}																			\
}

/*******************************************************************************
** T-FF2
********************************************************************************/
#define	TFF2_INIT0(init)														\
{																				\
		TFF2_Initialize(init);													\
		TFF2_Counter();															\
}

#define	TFF2_INIT																\
{																				\
		TFF2_Counter_Clear();													\
}

#define	TFF2_PROC(Reset, Tin, Out, Out2)										\
{																				\
	{																			\
		UCHAR	ucret;															\
		ucret = TFF2_Evaluate(Reset, Tin);										\
		Out = ucret & 0x01;														\
		Out2 = ~Out & 0x01;														\
		TFF2_Counter();															\
	}																			\
}

/*******************************************************************************
** D-FF2
********************************************************************************/
#define	DFF2_INIT0(init)														\
{																				\
		DFF2_Initialize(init);													\
		DFF2_Counter();															\
}

#define	DFF2_INIT																\
{																				\
		DFF2_Counter_Clear();													\
}

#define	DFF2_PROC(Din, Clk, Out, Out2)											\
{																				\
	{																			\
		UCHAR	ucret;															\
		ucret = DFF2_Evaluate(Din, Clk);										\
		Out = ucret & 0x01;														\
		Out2 = ~Out & 0x01;														\
		DFF2_Counter();															\
	}																			\
}

/*******************************************************************************
Typedef definitions
*******************************************************************************/

/*******************************************************************************
Imported global variables and functions (from other files)
*******************************************************************************/

/*******************************************************************************
Exported global variables and functions (to be accessed by other files)
*******************************************************************************/
/*	Function Prototype	*/
void	RSFF_Initialize( void );
void	RSFF_Counter_Clear( void );
UCHAR	RSFF_Proc( UCHAR Set, UCHAR Reset );
void	RSFF_Counter( void );

void	DFF_Initialize( void );
void	DFF_Counter_Clear( void );
UCHAR	DFF_Proc( UCHAR Din, UCHAR CKin );
void	DFF_Counter( void );

void	TFF_Initialize( void );
void	TFF_Counter_Clear( void );
UCHAR	TFF_Proc( UCHAR Tin, UCHAR Reset );
void	TFF_Counter( void );

void	JKFF_Initialize( UCHAR init );
void	JKFF_Counter_Clear( void );
UCHAR	JKFF_Evaluate( UCHAR Jin, UCHAR Clk,UCHAR Kin );
void	JKFF_Counter( void );

void	RSFF2_Initialize( UCHAR  init );
void	RSFF2_Counter_Clear( void );
UCHAR	RSFF2_Evaluate( UCHAR Sin, UCHAR Rin );
void	RSFF2_Counter( void );

void	TFF2_Initialize( UCHAR  init );
void	TFF2_Counter_Clear( void );
UCHAR	TFF2_Evaluate( UCHAR Reset, UCHAR Tin );
void	TFF2_Counter( void );

void	DFF2_Initialize( UCHAR init );
void	DFF2_Counter_Clear( void );
UCHAR	DFF2_Evaluate( UCHAR Din, UCHAR Clk );
void	DFF2_Counter( void );

#endif /* _FLIPFLOP_H_ */
