/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/

/*******************************************************************************
* File Name    : clk.c
* Version      : Applilet EZ PL for RL78
* Device(s)    : R5F1056A,R5F1057A,R5F1058A
* Tool-Chain   : CC-RL
* Description  : This file implements main function.
* Creation Date: 2016/09/14
*******************************************************************************/

/*******************************************************************************
Includes <System Includes> , "Project Includes"
*******************************************************************************/
#include "sfr.h"
#include "common.h"
#include "config.h"
#include <string.h>
#include "clk.h"

#if defined CLK_USED
/*******************************************************************************
Macro definitions
*******************************************************************************/

/*******************************************************************************
Typedef definitions
*******************************************************************************/

/*******************************************************************************
Imported global variables and functions (from other files)
*******************************************************************************/

/*******************************************************************************
Exported global variables and functions (to be accessed by other files)
*******************************************************************************/

/*******************************************************************************
Private variables and functions
*******************************************************************************/
static UCHAR	g_ucCLK_LastOut[(CLK_USED / 8) + 1];
static USHORT	g_ushCLK_ElapsedTime[CLK_USED];
static UCHAR	g_ucCLK_Count;

/*******************************************************************************
* Function Name: CLK_Initialize
* Description  : CLK_Initialize
* Arguments    : none
* Return Value : none
********************************************************************************/
void CLK_Initialize( void )
{
	/* memset( "The last output value of clock panel", 0, ("Number of clock panels" / 8 ) + 1 ) */
	memset(g_ucCLK_LastOut, 0, (CLK_USED / 8) + 1);

	/* memset( "Elapsed time of clock panel", 0, "Number of clock panels" * 2) */
	memset(g_ushCLK_ElapsedTime, 0, CLK_USED * 2);
}

/*******************************************************************************
* Function Name: CLK_Counter_Clear
* Description  : CLK_Counter_Clear
* Arguments    : none
* Return Value : none
********************************************************************************/
void CLK_Counter_Clear( void )
{
	/* "Clock panel count" <- 0 */
	g_ucCLK_Count = 0;
}

/*******************************************************************************
* Function Name: CLK_Evaluate
* Description  : CLK_Evaluate
* Arguments    : Sec     ... Wavy switch time(s)
*                MiliSec ... Wavy switch time(ms)
*                Trig    ... Trigger
* Return Value : The value set to the output terminal is returned.(0 or 1)
********************************************************************************/
UCHAR CLK_Evaluate( USHORT Sec, USHORT MiliSec, UCHAR Trig )
{
	UCHAR	ret;		/* Return value */
	UCHAR	lastclk;	/* Last output value of clock panel */
	USHORT	elapsedtime;/* Elapsed time of clock panel */
	UCHAR	nowclk;		/* Output value of clock panel */
	USHORT	time;		/* Wavy switch time */

	/* Last output value of clock panel is acquired. */
	lastclk = BitMemGet(g_ucCLK_LastOut, g_ucCLK_Count);

	/* Elapsed time of clock panel is acquired. */
	elapsedtime = g_ushCLK_ElapsedTime[g_ucCLK_Count];


	if ((0 <= Sec) && (Sec < 300)) {
		/* (Wavy switch time)<-(100 * "Wavy switch time"(s)) + "Wavy switch time"(ms)/10 */
		time = ((100 * Sec) + (MiliSec / 10));
	}
	else if ((300 <= Sec) && (Sec < 1200)) {
		/* (Wavy switch time)<-(20 * "Wavy switch time"(s)) + "Wavy switch time"(ms)/50 */
		time = ((20 * Sec) + (MiliSec / 50));
	}
	else {
		/* (Wavy switch time)<-"Wavy switch time"(s) */
		time = Sec;
	}

	if (Trig == HIGH) {
		/* (Elapsed time of clock panel)++ */
		elapsedtime++;
	}

	if (elapsedtime >= time) {
		/* (Return value)<-((Reversing of (Last output value of clock panel)) & 0x01) */
		ret = ((~lastclk) & 0x01);
		/* (Elapsed time of clock panel)<-0 */
		elapsedtime = 0;
	}
	else {
		/* (Return value)<-(Last output value of clock panel) */
		ret = lastclk;
	}

	/* This output value of clock panel is preserved. */
	BitMemSet(g_ucCLK_LastOut, g_ucCLK_Count, ret);

	/* Elapsed time of clock panel is preserved.  */
	g_ushCLK_ElapsedTime[g_ucCLK_Count] =elapsedtime;

	/* Return value is returned. */
	return ret;
}

/*******************************************************************************
* Function Name: CLK_Counter
* Description  : CLK_Counter
* Arguments    : none
* Return Value : none
********************************************************************************/
void CLK_Counter( void )
{
	/* "Clock panel count" ++ */
	g_ucCLK_Count++;
}
#endif /* end of defined CLK_USED */

