/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/

/*******************************************************************************
* File Name    : flipflop.c
* Version      : Applilet EZ PL for RL78
* Device(s)    : R5F1056A,R5F1057A,R5F1058A
* Tool-Chain   : CC-RL
* Description  : This file implements main function.
* Creation Date: 2016/09/14
*******************************************************************************/

/*******************************************************************************
Includes <System Includes> , "Project Includes"
*******************************************************************************/
#include "sfr.h"
#include "common.h"
#include "config.h"
#include <string.h>
#include "flipflop.h"

/*******************************************************************************
Macro definitions
*******************************************************************************/

/*******************************************************************************
Typedef definitions
*******************************************************************************/

/*******************************************************************************
Imported global variables and functions (from other files)
*******************************************************************************/

/*******************************************************************************
Exported global variables and functions (to be accessed by other files)
*******************************************************************************/

/*******************************************************************************
Private variables and functions
*******************************************************************************/
#if defined RS_FF_USED
static UCHAR	ucgRSFF_Last[(RS_FF_USED / 8) + 1];
static UCHAR	ucgRSFF_Cnt = 0;
#endif /* end of defined RS_FF_USED */

#if defined D_FF_USED
static UCHAR	ucgDFF_Last[(D_FF_USED / 8) + 1];
static UCHAR	ucgDFF_LastCK[D_FF_USED];
static UCHAR	ucgDFF_Cnt = 0;
#endif /* end of defined D_FF_USED */

#if defined T_FF_USED
static UCHAR	ucgTFF_Last[(T_FF_USED / 8) + 1];
static UCHAR	ucgTFF_LastTin[T_FF_USED];
static UCHAR	ucgTFF_Cnt = 0;
#endif /* end of defined T_FF_USED */

#if defined JKFF_USED
static UCHAR	g_ucJKFF_LastOut[(JKFF_USED / 8) + 1];
static UCHAR	g_ucJKFF_LastClk[(JKFF_USED / 8) + 1];
static UCHAR	g_ucJKFF_NextOut[(JKFF_USED / 8) + 1];
static UCHAR	g_ucJKFF_Count = 0;
#endif /* end of defined JKFF_USED */

#if defined RSFF2_USED
static UCHAR	g_ucRSFF2_LastOut[(RSFF2_USED / 8) + 1];
static UCHAR	g_ucRSFF2_Count = 0;
#endif /* end of defined RSFF2_USED */

#if defined TFF2_USED
static UCHAR	g_ucTFF2_LastOut[(TFF2_USED / 8) + 1];
static UCHAR	g_ucTFF2_LastTin[(TFF2_USED / 8) + 1];
static UCHAR	g_ucTFF2_NextOut[(TFF2_USED / 8) + 1];
static UCHAR	g_ucTFF2_Count = 0;
#endif /* end of defined TFF2_USED */

#if defined DFF2_USED
static UCHAR	g_ucDFF2_LastOut[(DFF2_USED / 8) + 1];
static UCHAR	g_ucDFF2_LastClk[(DFF2_USED / 8) + 1];
static UCHAR	g_ucDFF2_NextOut[(DFF2_USED / 8) + 1];
static UCHAR	g_ucDFF2_Count = 0;
#endif /* end of defined DFF2_USED */

#if defined RS_FF_USED
/*******************************************************************************
* Function Name: RSFF_Proc
* Description  : RS-type Flipflop.
* Arguments    : Set   		... S
*                Reset  	... R
* Return Value : out
********************************************************************************/
UCHAR RSFF_Proc( UCHAR Set, UCHAR Reset )
{
	UCHAR	ucret;
	
	if(Reset == LOW)
	{
		if(Set == HIGH)		ucret = HIGH;
		else if(Set == LOW)	ucret = BitMemGet(ucgRSFF_Last, ucgRSFF_Cnt);
	}
	else if(Reset == HIGH)	ucret = LOW;

	BitMemSet(ucgRSFF_Last, ucgRSFF_Cnt, ucret);

	return (ucret & 0x01);
}

/*******************************************************************************
* Function Name: RSFF_Initialize
* Description  : RS-type Flipflop Initialize.
* Arguments    : none
* Return Value : none
********************************************************************************/
void RSFF_Initialize( void )
{
	memset(ucgRSFF_Last, 0, (RS_FF_USED / 8) + 1);
}

/*******************************************************************************
* Function Name: RSFF_Counter_Clear
* Description  : RS-type Flipflop Counter Clear.
* Arguments    : none
* Return Value : none
********************************************************************************/
void RSFF_Counter_Clear( void )
{
	ucgRSFF_Cnt = 0;
}

/*******************************************************************************
* Function Name: RSFF_Counter
* Description  : RS-type Flipflop Counter.
* Arguments    : none
* Return Value : none
********************************************************************************/
void RSFF_Counter( void )
{
	ucgRSFF_Cnt ++;
}
#endif /* end of defined RS_FF_USED */

#if defined D_FF_USED
/*******************************************************************************
* Function Name: DFF_Proc
* Description  : D-type Flipflop.
* Arguments    : Din   ... D
*                CKin  ... C
* Return Value : out
********************************************************************************/
UCHAR DFF_Proc( UCHAR Din, UCHAR CKin )
{
	UCHAR	ucret;
	UCHAR	uclastck;

	ucret = BitMemGet(ucgDFF_Last, ucgDFF_Cnt);
	
	if(CKin == LOW)
	{
		uclastck = 0;
	}
	else if(CKin == HIGH)
	{
		uclastck = ByteMemGet(ucgDFF_LastCK, ucgDFF_Cnt);
		switch(uclastck)
		{
			case 0:
				uclastck = Din | 0x80;
				break;
			case 1:
				break;
			default:
				ucret = uclastck & 0x01;
				uclastck = 1;
				break;
		}
	}

	BitMemSet(ucgDFF_Last, ucgDFF_Cnt, ucret);
	ByteMemSet(ucgDFF_LastCK, ucgDFF_Cnt, uclastck);

	return (ucret & 0x01);
}

/*******************************************************************************
* Function Name: DFF_Initialize
* Description  : D-type Flipflop Initialize.
* Arguments    : none
* Return Value : none
********************************************************************************/
void DFF_Initialize( void )
{
	memset(ucgDFF_Last, 0, (D_FF_USED / 8) + 1);
	memset(ucgDFF_LastCK, 0, D_FF_USED);
}

/*******************************************************************************
* Function Name: DFF_Counter_Clear
* Description  : D-type Flipflop Panel Counter Clear.
* Arguments    : none
* Return Value : none
********************************************************************************/
void DFF_Counter_Clear( void )
{
	ucgDFF_Cnt = 0;
}

/*******************************************************************************
* Function Name: DFF_Counter
* Description  : D-type Flipflop Panel Counter.
* Arguments    : none
* Return Value : none
********************************************************************************/
void DFF_Counter( void )
{
	ucgDFF_Cnt ++;
}
#endif /* end of defined D_FF_USED */

#if defined T_FF_USED
/*******************************************************************************
* Function Name: TFF_Proc
* Description  : T-type Flipflop.
* Arguments    : Tin   	... T
*                Reset	... R
* Return Value : out
********************************************************************************/
UCHAR TFF_Proc( UCHAR Tin, UCHAR Reset )
{
	UCHAR	ucret;
	UCHAR	uclastin;
	
	if(Reset == HIGH)
	{
		uclastin = 0x40;
		ucret = LOW;
	}
	else if(Reset == LOW)
	{
		ucret = BitMemGet(ucgTFF_Last, ucgTFF_Cnt);
		if(Tin == LOW)
		{
			uclastin = LOW;
		}
		else if(Tin == HIGH)
		{
			uclastin = ByteMemGet(ucgTFF_LastTin, ucgTFF_Cnt);
			switch(uclastin)
			{
				case 0x00:
					uclastin = 0x80;
					break;
				case 0x40:
					ucret = 0;
					break;
				case 0x80:
					uclastin = 1;
					if(ucret == HIGH)		ucret = LOW;
					else if(ucret == LOW)	ucret = HIGH;
					break;
			}
		}
	}

	BitMemSet(ucgTFF_Last, ucgTFF_Cnt, ucret);
	ByteMemSet(ucgTFF_LastTin, ucgTFF_Cnt, uclastin);

	return(ucret & 0x01);
}	

/*******************************************************************************
* Function Name: TFF_Initialize
* Description  : T-type Flipflop Initialize.
* Arguments    : none
* Return Value : none
********************************************************************************/
void TFF_Initialize( void )
{
	memset(ucgTFF_Last, 0, (T_FF_USED / 8) + 1);
	memset(ucgTFF_LastTin, 0, T_FF_USED);
}

/*******************************************************************************
* Function Name: TFF_Counter_Clear
* Description  : T-type Flipflop Panel Counter Clear.
* Arguments    : none
* Return Value : none
********************************************************************************/
void TFF_Counter_Clear( void )
{
	ucgTFF_Cnt = 0;
}

/*******************************************************************************
* Function Name: TFF_Counter
* Description  : T-type Flipflop Panel Counter.
* Arguments    : none
* Return Value : none
********************************************************************************/
void TFF_Counter( void )
{
	ucgTFF_Cnt ++;
}
#endif /* end of defined T_FF_USED */

#if defined JKFF_USED
/*******************************************************************************
* Function Name: JKFF_Initialize
* Description  : JKFF_Initialize for JK-FF panel.
* Arguments    : init      ... Initial value of the last output value of JK-FF
* Return Value : none
********************************************************************************/
void JKFF_Initialize( UCHAR init )
{
	/* BitMemSet( "Last output value of JK-FF", "Number of JK-FF panels", init) */
	BitMemSet(g_ucJKFF_LastOut, g_ucJKFF_Count, init);
	
	/* BitMemSet( "Last value of JK-FFClk", "Number of JK-FF panels", 0) */
	BitMemSet(g_ucJKFF_LastClk, g_ucJKFF_Count, 0);
	
	/* BitMemSet( "Next output value of JK-FF", "Number of JK-FF panels", init ) */
	BitMemSet(g_ucJKFF_NextOut, g_ucJKFF_Count, init );
}

/*******************************************************************************
* Function Name: JKFF_Counter_Clear
* Description  : JKFF_Counter_Clear for JK-FF panel.
* Arguments    : none
* Return Value : none
********************************************************************************/
void JKFF_Counter_Clear( void )
{
	/* "Count value of JK-FF panel" <- 0 */
	g_ucJKFF_Count = 0;
}

/*******************************************************************************
* Function Name: JKFF_Evaluate
* Description  : Value set to output terminal Q is returned.(0 or 1).
* Arguments    : Jin      ... Input value of terminal J
*                Clk      ... Input value of terminal Clk
*                Kin      ... Input value of terminal K
* Return Value : Next output value
********************************************************************************/
UCHAR JKFF_Evaluate( UCHAR Jin, UCHAR Clk,UCHAR Kin )
{
	UCHAR	ret;		/* Return value */
	UCHAR	Next;		/* Next output value */
	UCHAR	Last;		/* Last output value */
	UCHAR	LastClk;	/* Last clock value */
	
	/* Next output value of JK-FF is acquired. */
	ret = BitMemGet(g_ucJKFF_NextOut, g_ucJKFF_Count);
	
	/* Last output value is acquired. */
	Last = BitMemGet(g_ucJKFF_LastOut, g_ucJKFF_Count); 
	
	/* Last Clk value is acquired. */
	LastClk = BitMemGet(g_ucJKFF_LastClk, g_ucJKFF_Count);
	
	if (LastClk == LOW) {
		if (Clk == HIGH) {
			if ((Jin & Kin) == LOW ) {
				if ((Jin | Kin) == HIGH) {
					/* (Next output value)<-(Jin) */
					Next = Jin;
				}
				else {
					/* (Next output value)<-(Return value) */
					Next = ret;
				}
			}
			else {
				/* (Next output value)<-(Reversing of (Return value)) */
				Next = ~ret;
				/* (Next output value)&0x01 */
				Next = Next & 0x01;
			}
		}
		else {
			/* (Next output value)<-(Return value) */
			Next = ret;
		}
	}
	else {
		/* (Next output value)<-(Return value) */
		Next = ret;
	}

	/* This Clk value is preserved. */
	BitMemSet(g_ucJKFF_LastClk, g_ucJKFF_Count, Clk);

	/* This output value is preserved. */
	BitMemSet(g_ucJKFF_LastOut, g_ucJKFF_Count, ret);

	/* Next output value is preserved. */
	BitMemSet(g_ucJKFF_NextOut, g_ucJKFF_Count, Next);
	
	/* Return value is returned. */
	return ret;
}

/*******************************************************************************
* Function Name: JKFF_Counter
* Description  : JKFF_Counter for JK-FF panel.
* Arguments    : none
* Return Value : none
********************************************************************************/
void JKFF_Counter( void )
{
	/* "Count value of JK-FF panel"++ */
	g_ucJKFF_Count++;
}
#endif /* end of defined JKFF_USED */

#if defined RSFF2_USED
/*******************************************************************************
* Function Name: RSFF2_Initialize
* Description  : RSFF2_Initialize for RS-FF2 panel.
* Arguments    : init      ... Initial value of the last output value of RS-FF2
* Return Value : none
********************************************************************************/
void RSFF2_Initialize( UCHAR  init )
{
	/* BitMemSet( "Last output value of RS-FF2", "Count of RS-FF2 panel", init ) */
	BitMemSet(g_ucRSFF2_LastOut, g_ucRSFF2_Count, init );
}

/*******************************************************************************
* Function Name: RSFF2_Counter_Clear
* Description  : RSFF2_Counter_Clear RS-FF2 panel.
* Arguments    : none
* Return Value : none
********************************************************************************/
void RSFF2_Counter_Clear( void )
{
	/* "Count value of RS-FF2 panel" <- 0 */
	g_ucRSFF2_Count = 0;
}

/*******************************************************************************
* Function Name: RSFF2_Evaluate
* Description  : RSFF2_Evaluate for RS-FF2 panel.
* Arguments    : Sin      ... Input value of terminal S
*                Rin      ... Input value of terminal R
* Return Value : Value set to output terminal Q is returned.(0 or 1)
********************************************************************************/
UCHAR RSFF2_Evaluate( UCHAR Sin, UCHAR Rin )
{
	UCHAR	ucret; /* Return value */
	
	if (Rin == LOW) {
		if (Sin == LOW) {
			/* Last output value is acquired. */
			ucret = BitMemGet(g_ucRSFF2_LastOut, g_ucRSFF2_Count);
		}
		else {
			/* (Return value)<-High */
			ucret = HIGH;
		}
	}
	else {
		/* (Return value)<-Low */
		ucret = LOW;
	}
	
	/* This output value is set. */
	BitMemSet(g_ucRSFF2_LastOut, g_ucRSFF2_Count,ucret);
	
	/* return ((Return value)&0x01) */
	return (ucret & 0x01);
}

/*******************************************************************************
* Function Name: RSFF2_Counter
* Description  : RSFF2_Counter RS-FF2 panel.
* Arguments    : none
* Return Value : none
********************************************************************************/
void RSFF2_Counter( void )
{
	/* "Count value of RS-FF2 panel"++ */
	g_ucRSFF2_Count++;
}
#endif /* end of defined RSFF2_USED */

#if defined TFF2_USED
/*******************************************************************************
* Function Name: TFF2_Initialize
* Description  : TFF2_Initialize for T-FF2 panel.
* Arguments    : init      ... Initial value of the last output value of T-FF2
* Return Value : none
********************************************************************************/
void TFF2_Initialize( UCHAR  init )
{
	/* BitMemSet( "Last output value of T-FF2", "Count value of T-FF2 panel", init) */
	BitMemSet(g_ucTFF2_LastOut, g_ucTFF2_Count, init);
	
	/* BitMemSet( "Last value of input T of T-FF2", "Count value of T-FF2 panel", 0) */
	BitMemSet(g_ucTFF2_LastTin, g_ucTFF2_Count, 0);
	
	/* BitMemSet( "Next output value of T-FF2", "Count value of T-FF2 panel", init) */
	BitMemSet(g_ucTFF2_NextOut, g_ucTFF2_Count, init);
}

/*******************************************************************************
* Function Name: TFF2_Counter_Clear
* Description  : TFF2_Counter_Clear for T-FF2 panel.
* Arguments    : none
* Return Value : none
********************************************************************************/
void TFF2_Counter_Clear( void )
{
	/* "Count value of T-FF2 panel" <- 0 */
	g_ucTFF2_Count = 0;
}

/*******************************************************************************
* Function Name: TFF2_Evaluate
* Description  : TFF2_Evaluate for T-FF2 panel.
* Arguments    : Reset      ... Input value of reset terminal
*                Tin        ... Input value of terminal T
* Return Value : Value set to output terminal Q is returned.(0 or 1)
********************************************************************************/
UCHAR TFF2_Evaluate( UCHAR Reset, UCHAR Tin )
{
	UCHAR	uclastin;	/* Input value of last terminal T */
	UCHAR	ucret;		/* Return value */
	UCHAR	Last;		/* Last output value */
	UCHAR	Next;		/* Next output value */
	
	/* Next output value of T-FF2 is acquired. */
	ucret = BitMemGet(g_ucTFF2_NextOut, g_ucTFF2_Count);
	
	/* Last output value is acquired. */
	Last = BitMemGet(g_ucTFF2_LastOut, g_ucTFF2_Count);
	
	/* Last terminal T input value is acquired. */
	uclastin = BitMemGet(g_ucTFF2_LastTin, g_ucTFF2_Count);
	
	if (Reset == LOW) {
		if (uclastin == LOW) {
			if (Tin == HIGH) {
				/* (Next output value)<-(Reversing of (Return value)) */
				Next = (~ucret) & 0x01;
			}
			else {
				/* (Next output value)<-(Return value) */
				Next = ucret;
			}
		}
		else {
			/* (Next output value)<-(Return value) */
			Next = ucret;
		}
	}
	else {
		/* (Next output value)<-Low */
		Next = LOW;
	}
	
	/* This terminal T input value is preserved. */
	BitMemSet(g_ucTFF2_LastTin, g_ucTFF2_Count, Tin);
	
	/* This output value is preserved. */
	BitMemSet(g_ucTFF2_LastOut, g_ucTFF2_Count, ucret);
	
	/* Next output value is preserved. */
	BitMemSet(g_ucTFF2_NextOut, g_ucTFF2_Count, Next);
	
	/* Return value is returned. */
	return ucret;
}

/*******************************************************************************
* Function Name: TFF2_Counter
* Description  : TFF2_Counter for T-FF2 panel.
* Arguments    : none
* Return Value : none
********************************************************************************/
void TFF2_Counter( void )
{
	/* "Count value of T-FF2 panel"++ */
	g_ucTFF2_Count++;
}
#endif /* end of defined TFF2_USED */

#if defined DFF2_USED
/*******************************************************************************
* Function Name: DFF2_Initialize
* Description  : DFF2_Initialize for D-FF2 panel.
* Arguments    : init      ... Initial value of the last output value of D-FF2
* Return Value : none
********************************************************************************/
void DFF2_Initialize( UCHAR init )
{
	/* BitMemSet( "last output value of D-FF2", "Count value of D-FF2 panel", init) */
	BitMemSet(g_ucDFF2_LastOut, g_ucDFF2_Count, init);
	
	/* BitMemSet( "Last value of Clk of D-FF2", "Count value of D-FF2 panel", 0) */
	BitMemSet(g_ucDFF2_LastClk, g_ucDFF2_Count, 0);
	
	/* BitMemSet( "Next output value of D-FF2", "Count value of D-FF2 panel", init) */
	BitMemSet(g_ucDFF2_NextOut, g_ucDFF2_Count, init);
}

/*******************************************************************************
* Function Name: DFF2_Counter_Clear
* Description  : DFF2_Counter_Clear for D-FF2 panel.
* Arguments    : none
* Return Value : none
********************************************************************************/
void DFF2_Counter_Clear( void )
{
	/* "Count value of D-FF2 panel" <- 0 */
	g_ucDFF2_Count = 0;
}

/*******************************************************************************
* Function Name: DFF2_Evaluate
* Description  : DFF2_Evaluate for D-FF2 panel.
* Arguments    : Din      ... Input value of terminal D
*                Clk      ... Input value of clock terminal
* Return Value : Value set to output terminal Q is returned.(0 or 1)
********************************************************************************/
UCHAR DFF2_Evaluate( UCHAR Din, UCHAR Clk )
{
	UCHAR	uclastck;	/* Last value of Clk of D-FF2 */
	UCHAR	ucret;		/* Return value */
	UCHAR	Next;		/* Next output value */
	UCHAR	Last;		/* Last output value */
	
	/* Next output value of D-FF2 is acquired. */
	ucret = BitMemGet(g_ucDFF2_NextOut, g_ucDFF2_Count);
	
	/* Last output value is acquired. */
	Last = BitMemGet(g_ucDFF2_LastOut, g_ucDFF2_Count);
	
	/* Last Clk value is acquired. */
	uclastck = BitMemGet(g_ucDFF2_LastClk, g_ucDFF2_Count);
	
	if (uclastck == LOW) {
		if (Clk == HIGH) {
			/* (Next output value)<-(Input value of terminal D) */
			Next = Din;
		}
		else {
			/* (Next output value)<-(Return value) */
			Next = ucret;
		}
	}
	else {
		/* (Next output value)<-(Return value) */
		Next = ucret;
	}
	
	/* This Clk value is preserved. */
	BitMemSet(g_ucDFF2_LastClk, g_ucDFF2_Count, Clk);
	
	/* This output value is preserved. */
	BitMemSet(g_ucDFF2_LastOut, g_ucDFF2_Count, ucret);
	
	/* Next output value is preserved. */
	BitMemSet(g_ucDFF2_NextOut, g_ucDFF2_Count, Next);
	
	/* Return value is returned. */
	return ucret;
}

/*******************************************************************************
* Function Name: DFF2_Counter
* Description  : DFF2_Counter for D-FF2 panel.
* Arguments    : none
* Return Value : none
********************************************************************************/
void DFF2_Counter( void )
{
	/* "Count value of D-FF2 panel" ++ */
	g_ucDFF2_Count++;
}
#endif /* end of defined DFF2_USED */

